/* read_malign.h - shared procedures for handling textual multiple alignments */

#include "util.h"
#include "read_malign.h"

int *position_in_seq1(char **A, int ncol)
{
	int k, j, *col2pos = ckalloc(ncol*sizeof(int));

        for (k = j = 0; j < ncol; ++j)
                col2pos[j] = (isalpha(A[0][j]) ? ++k : 0);
	return col2pos;
}

char **read_malign(char *filename, int *nrow_ptr, int *ncol_ptr,
  char ***seqname_ptr)
{
	int nrow, ncol, i, j, c;
	char **A, **seqname, buf[500], *s;
	FILE *fp = ckopen(filename, "r");

	if (fgets(buf, 500, fp) == NULL ||
	    sscanf(buf, "%d %d", &nrow, &ncol) != 2)
		fatal("Cannot determine numbers of rows and columns.");
	seqname = ckalloc(nrow * sizeof(char *));
	for (i = 0; i < nrow; ++i) {
		if (fgets(buf, 500, fp) == NULL)
			fatal("Cannot determine sequence names.");
		if ((s = strchr(buf, '\n')) != NULL)
			*s = '\0';
		if ((s = strchr(buf, '\r')) != NULL)
			*s = '\0';
		seqname[i] = copy_string(buf);
	}
	A = ckalloc(nrow * sizeof(char *));
	A[0] = ckalloc(nrow * ncol * sizeof(char));
	for (i = 1; i < nrow; ++i)
		A[i] = A[i-1] + ncol*sizeof(char);
	for (i = 0; i < nrow; ++i) {
		for (j = 0; j < ncol; ) {
			if ((c = fgetc(fp)) == EOF)
				fatal("Alignment file is incomplete.");
			if (c != '\n' && c != '\r')
				A[i][j++] = c;
		}
	}
	while ((c = fgetc(fp)) != EOF)
		if (strchr("ACGT-", c))
			fatal("Too many letters in the alignment file.");
	*nrow_ptr = nrow;
	*ncol_ptr = ncol;
	*seqname_ptr = seqname;
	return A;
}
